"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OracleSqlBuilder = void 0;
const SqlBuilder_1 = require("./SqlBuilder");
const TypeAdapter_1 = require("../TypeAdapter");
const values_1 = require("../expressions/values");
const AbstractSqlBuilder_1 = require("./AbstractSqlBuilder");
const Column_1 = require("../utils/Column");
const values_2 = require("../expressions/values");
class OracleSqlBuilder extends AbstractSqlBuilder_1.AbstractSqlBuilder {
    constructor() {
        super();
        this.oracle = true;
        this._insertSupportWith = false;
        this._supportTableAliasWithAs = false;
        this._trueValue = '1';
        this._falseValue = '0';
        this._trueValueForCondition = '(1=1)';
        this._falseValueForCondition = '(0=1)';
        this._nullValueForCondition = '(0=null)';
        this._operationsThatNeedParenthesis._getTime = true;
        this._operationsThatNeedParenthesis._getMonth = true;
        this._operationsThatNeedParenthesis._getDay = true;
        this._operationsThatNeedParenthesis._negate = true;
    }
    _getUuidStrategy() {
        return this._connectionConfiguration.uuidStrategy || 'custom-functions';
    }
    _isReservedKeyword(word) {
        return word.toUpperCase() in reservedWords;
    }
    _nextSequenceValue(_params, sequenceName) {
        return this._escape(sequenceName, false) + '.nextval';
    }
    _currentSequenceValue(_params, sequenceName) {
        return this._escape(sequenceName, false) + '.currval';
    }
    _fromNoTable() {
        return ' from dual';
    }
    _appendSql(value, params) {
        if ((0, values_1.isValueSource)(value) && !(0, Column_1.isColumn)(value)) {
            const valueSourcePrivate = (0, values_2.__getValueSourcePrivate)(value);
            if (valueSourcePrivate.__isBooleanForCondition) {
                if (valueSourcePrivate.__optionalType === 'required') {
                    return 'case when ' + super._appendConditionSql(value, params) + ' then 1 else 0 end';
                }
                else {
                    return 'case when ' + super._appendConditionSql(value, params) + ' then 1 when not ' + super._appendConditionSqlParenthesis(value, params) + ' then 0 else null end';
                }
            }
        }
        return super._appendSql(value, params);
    }
    _appendConditionSql(value, params) {
        if ((0, values_1.isValueSource)(value) && !(0, Column_1.isColumn)(value) && (0, SqlBuilder_1.hasToSql)(value)) {
            const valueSourcePrivate = (0, values_2.__getValueSourcePrivate)(value);
            if ((0, values_1.__isBooleanValueSource)(valueSourcePrivate) && !valueSourcePrivate.__isBooleanForCondition) {
                const sql = value.__toSqlForCondition(this, params);
                if (!sql || sql === this._trueValueForCondition || sql === this._falseValueForCondition) {
                    return sql;
                }
                else {
                    return '(' + sql + ' = 1)';
                }
            }
        }
        return super._appendConditionSql(value, params);
    }
    _appendConditionParam(value, params, columnType, columnTypeName, typeAdapter, forceTypeCast) {
        if ((0, values_1.__isBooleanValueType)(columnType)) {
            if ((0, Column_1.isColumn)(value)) {
                const columnPrivate = (0, Column_1.__getColumnPrivate)(value);
                const typeAdapter = columnPrivate.__typeAdapter;
                if (typeAdapter instanceof TypeAdapter_1.CustomBooleanTypeAdapter) {
                    return '(' + this._appendRawColumnName(value, params) + ' = ' + this._appendLiteralValue(typeAdapter.trueValue, params) + ')';
                }
            }
            return '(' + this._appendParam(value, params, columnType, columnTypeName, typeAdapter, forceTypeCast) + ' = 1)';
        }
        return this._appendParam(value, params, columnType, columnTypeName, typeAdapter, forceTypeCast);
    }
    _appendColumnName(column, params) {
        const columnPrivate = (0, Column_1.__getColumnPrivate)(column);
        const typeAdapter = columnPrivate.__typeAdapter;
        if ((0, values_1.__isBooleanValueSource)(columnPrivate)) {
            if (typeAdapter instanceof TypeAdapter_1.CustomBooleanTypeAdapter) {
                if (columnPrivate.__optionalType === 'required') {
                    return 'case when ' + this._appendRawColumnName(column, params) + ' = ' + this._appendLiteralValue(typeAdapter.trueValue, params) + ' then 1 else 0 end';
                }
                else {
                    return 'case ' + this._appendRawColumnName(column, params) + ' when ' + this._appendLiteralValue(typeAdapter.trueValue, params) + ' then 1 when ' + this._appendLiteralValue(typeAdapter.falseValue, params) + ' then 0 else null end';
                }
            }
        }
        return this._appendRawColumnName(column, params);
    }
    _appendColumnNameForCondition(column, params) {
        const columnPrivate = (0, Column_1.__getColumnPrivate)(column);
        const typeAdapter = columnPrivate.__typeAdapter;
        if ((0, values_1.__isBooleanValueSource)(columnPrivate)) {
            if (typeAdapter instanceof TypeAdapter_1.CustomBooleanTypeAdapter) {
                return '(' + this._appendRawColumnName(column, params) + ' = ' + this._appendLiteralValue(typeAdapter.trueValue, params) + ')';
            }
            else {
                return '(' + this._appendRawColumnName(column, params) + ' = 1)';
            }
        }
        return this._appendRawColumnName(column, params);
    }
    _inlineSelectAsValueForCondition(query, params) {
        if (query.__oneColumn) {
            const columns = query.__columns;
            for (const prop in columns) {
                const column = columns[prop];
                if ((0, values_1.isValueSource)(column) && (0, values_1.__isBooleanValueSource)((0, values_2.__getValueSourcePrivate)(column))) {
                    return '((' + this._buildInlineSelect(query, params) + ') = 1)';
                }
                else {
                    return this._buildInlineSelect(query, params);
                }
            }
        }
        return this._buildInlineSelect(query, params);
    }
    _appendCustomBooleanRemapForColumnIfRequired(column, value, params) {
        const columnPrivate = (0, Column_1.__getColumnPrivate)(column);
        const columnTypeAdapter = columnPrivate.__typeAdapter;
        const columnTypeName = columnPrivate.__valueTypeName;
        const columnType = columnPrivate.__valueType;
        if (!(0, values_1.__isBooleanValueSource)(columnPrivate)) {
            return null; // non-boolean
        }
        if (columnTypeAdapter instanceof TypeAdapter_1.CustomBooleanTypeAdapter) {
            if ((0, Column_1.isColumn)(value)) {
                const valuePrivate = (0, Column_1.__getColumnPrivate)(value);
                const valueTypeAdapter = valuePrivate.__typeAdapter;
                if (valueTypeAdapter instanceof TypeAdapter_1.CustomBooleanTypeAdapter) {
                    if (columnTypeAdapter.trueValue === valueTypeAdapter.trueValue && columnTypeAdapter.falseValue === valueTypeAdapter.falseValue) {
                        return this._appendRawColumnName(value, params); // same boolean as column
                    }
                    if (columnPrivate.__optionalType === 'required') {
                        // remapped
                        return 'case when ' + this._appendRawColumnName(value, params) + ' = ' + this._appendLiteralValue(valueTypeAdapter.trueValue, params) + ' then ' + this._appendLiteralValue(columnTypeAdapter.trueValue, params) + ' else ' + this._appendLiteralValue(columnTypeAdapter.falseValue, params) + ' end';
                    }
                    else {
                        // remapped
                        return 'case ' + this._appendRawColumnName(value, params) + ' when ' + this._appendLiteralValue(valueTypeAdapter.trueValue, params) + ' then ' + this._appendLiteralValue(columnTypeAdapter.trueValue, params) + ' when ' + this._appendLiteralValue(valueTypeAdapter.falseValue, params) + ' then ' + this._appendLiteralValue(columnTypeAdapter.falseValue, params) + ' else null end';
                    }
                }
                else {
                    if (columnPrivate.__optionalType === 'required') {
                        // remapped
                        return 'case when ' + this._appendConditionValue(value, params, columnType, columnTypeName, columnTypeAdapter) + ' then ' + this._appendLiteralValue(columnTypeAdapter.trueValue, params) + ' else ' + this._appendLiteralValue(columnTypeAdapter.falseValue, params) + ' end';
                    }
                    else {
                        // remapped
                        return 'case ' + this._appendValue(value, params, columnType, columnTypeName, columnTypeAdapter) + ' when ' + this._trueValue + ' then ' + this._appendLiteralValue(columnTypeAdapter.trueValue, params) + ' when ' + this._falseValue + ' then ' + this._appendLiteralValue(columnTypeAdapter.falseValue, params) + ' else null end';
                    }
                }
            }
            else if ((0, values_1.isValueSource)(value)) {
                // There are some boolean expressions involved
                if (columnPrivate.__optionalType === 'required') {
                    // remapped
                    return 'case when ' + this._appendConditionValue(value, params, columnType, columnTypeName, columnTypeAdapter) + ' then ' + this._appendLiteralValue(columnTypeAdapter.trueValue, params) + ' else ' + this._appendLiteralValue(columnTypeAdapter.falseValue, params) + ' end';
                }
                else {
                    // remapped
                    return 'case when ' + this._appendConditionValue(value, params, columnType, columnTypeName, columnTypeAdapter) + ' then ' + this._appendLiteralValue(columnTypeAdapter.trueValue, params) + ' when not ' + this._appendConditionValueParenthesis(value, params, columnType, columnTypeName, columnTypeAdapter) + ' then ' + this._appendLiteralValue(columnTypeAdapter.falseValue, params) + ' else null end';
                }
            }
            else {
                if (columnPrivate.__optionalType === 'required') {
                    // remapped
                    return 'case when ' + this._appendConditionValue(value, params, columnType, columnTypeName, columnTypeAdapter) + ' then ' + this._appendLiteralValue(columnTypeAdapter.trueValue, params) + ' else ' + this._appendLiteralValue(columnTypeAdapter.falseValue, params) + ' end';
                }
                else if (value === true || value === false) {
                    // remapped
                    return 'case ' + this._appendValue(value, params, columnType, columnTypeName, columnTypeAdapter) + ' when ' + this._trueValue + ' then ' + this._appendLiteralValue(columnTypeAdapter.trueValue, params) + ' when ' + this._falseValue + ' then ' + this._appendLiteralValue(columnTypeAdapter.falseValue, params) + ' else null end';
                }
                else {
                    // remapped
                    // Oracle detects wrongly the value type in this case (when value is null), for this reason a cast in injected
                    return 'case cast(' + this._appendValue(value, params, columnType, columnTypeName, columnTypeAdapter) + ' as number) when ' + this._trueValue + ' then ' + this._appendLiteralValue(columnTypeAdapter.trueValue, params) + ' when ' + this._falseValue + ' then ' + this._appendLiteralValue(columnTypeAdapter.falseValue, params) + ' else null end';
                }
            }
        }
        // if value is column and its type adapter is CustomBooleanTypeAdapter append value will be required to normalize value
        // if not it is same boolean, nothing to transform here
        return null;
    }
    _appendWithColumns(withData, params) {
        if (withData.__selectData.__type === 'plain') {
            return '';
        }
        const columns = {};
        (0, SqlBuilder_1.flattenQueryColumns)(withData.__selectData.__columns, columns, '');
        let result = '';
        for (const property in columns) {
            if (result) {
                result += ', ';
            }
            result += this._appendColumnAlias(property, params);
        }
        return '(' + result + ')';
    }
    _appendWithKeyword(_recursive) {
        // Oracle doesn't uses the recursive keyword
        return 'with';
    }
    _appendColumnAlias(name, params) {
        if (!this._isWithGeneratedFinished(params)) {
            // Avoid quote identifiers when the with clause is generated
            return this._escape(name, true);
        }
        if (this._isAggregateArrayWrapped(params)) {
            // Avoid quote identifiers when the aggregate array query wrapper is generated
            return this._escape(name, true);
        }
        // Avoid automatically uppercase identifiers by oracle
        return this._forceAsIdentifier(name);
    }
    _appendParam(value, params, columnType, columnTypeName, typeAdapter, forceTypeCast) {
        if ((0, values_1.__isUuidValueType)(columnType) && this._getUuidStrategy() === 'custom-functions') {
            return 'uuid_to_raw(' + super._appendParam(value, params, columnType, columnTypeName, typeAdapter, forceTypeCast) + ')';
        }
        return super._appendParam(value, params, columnType, columnTypeName, typeAdapter, forceTypeCast);
    }
    _appendColumnValue(value, params, isOutermostQuery) {
        if (isOutermostQuery && this._getUuidStrategy() === 'custom-functions') {
            if ((0, values_1.__isUuidValueSource)((0, values_2.__getValueSourcePrivate)(value))) {
                return 'raw_to_uuid(' + this._appendSql(value, params) + ')';
            }
        }
        return this._appendSql(value, params);
    }
    _asString(params, valueSource) {
        // Transform an uuid to string
        if (this._getUuidStrategy() === 'string') {
            // No conversion required
            return this._appendSql(valueSource, params);
        }
        return 'raw_to_uuid(' + this._appendSql(valueSource, params) + ')';
    }
    _appendCompoundOperator(compoundOperator, _params) {
        switch (compoundOperator) {
            case 'union':
                return ' union ';
            case 'unionAll':
                return ' union all ';
            case 'intersect':
                return ' intersect ';
            case 'intersectAll':
                return ' intersect all ';
            case 'except':
                return ' minus ';
            case 'exceptAll':
                return ' minus all ';
            case 'minus':
                return ' except ';
            case 'minusAll':
                return ' except all ';
            default:
                throw new Error('Invalid compound operator: ' + compoundOperator);
        }
    }
    _buildSelectWithColumnsInfoForCompound(query, params, columnsForInsert, isOutermostQuery) {
        const result = this._buildSelectWithColumnsInfo(query, params, columnsForInsert, isOutermostQuery);
        if (query.__limit !== undefined || query.__offset !== undefined || query.__orderBy || query.__customization?.beforeOrderByItems || query.__customization?.afterOrderByItems) {
            return 'select * from (' + result + ')';
        }
        return result;
    }
    _buildSelectOrderBy(query, params) {
        if (query.__type === 'plain') {
            return super._buildSelectOrderBy(query, params);
        }
        const orderBy = query.__orderBy;
        if (!orderBy) {
            let orderByColumns = '';
            const customization = query.__customization;
            if (customization && customization.beforeOrderByItems) {
                orderByColumns += this._appendRawFragment(customization.beforeOrderByItems, params);
            }
            if (customization && customization.afterOrderByItems) {
                if (orderByColumns) {
                    orderByColumns += ', ';
                }
                orderByColumns += this._appendRawFragment(customization.afterOrderByItems, params);
            }
            if (!orderByColumns) {
                return '';
            }
            if (query.__orderingSiblingsOnly) {
                // Oracle recursive
                return ' order siblings by ' + orderByColumns;
            }
            else {
                return ' order by ' + orderByColumns;
            }
        }
        const columns = {};
        (0, SqlBuilder_1.flattenQueryColumns)(query.__columns, columns, '');
        const columnNames = Object.getOwnPropertyNames(columns);
        let orderByColumns = '';
        const customization = query.__customization;
        if (customization && customization.beforeOrderByItems) {
            orderByColumns += this._appendRawFragment(customization.beforeOrderByItems, params);
        }
        for (const entry of orderBy) {
            if (orderByColumns) {
                orderByColumns += ', ';
            }
            const order = entry.order;
            if (!order) {
                orderByColumns += this._appendCompoundOrderByColumnAlias(entry, columnNames, query, params);
            }
            else
                switch (order) {
                    case 'asc':
                    case 'desc':
                    case 'asc nulls first':
                    case 'asc nulls last':
                    case 'desc nulls first':
                    case 'desc nulls last':
                        orderByColumns += this._appendCompoundOrderByColumnAlias(entry, columnNames, query, params) + ' ' + order;
                        break;
                    case 'insensitive':
                    case 'asc insensitive':
                    case 'desc insensitive':
                    case 'asc nulls first insensitive':
                    case 'asc nulls last insensitive':
                    case 'desc nulls first insensitive':
                    case 'desc nulls last insensitive': {
                        let sqlOrder = order.substring(0, order.length - 12);
                        if (sqlOrder) {
                            sqlOrder = ' ' + sqlOrder;
                        }
                        orderByColumns += this._appendCompoundOrderByColumnAliasInsensitive(entry, columnNames, query, params) + sqlOrder;
                        break;
                    }
                    default:
                        throw new Error('Invalid order by: ' + order);
                }
        }
        if (customization && customization.afterOrderByItems) {
            if (orderByColumns) {
                orderByColumns += ', ';
            }
            orderByColumns += this._appendRawFragment(customization.afterOrderByItems, params);
        }
        if (!orderByColumns) {
            return '';
        }
        if (query.__orderingSiblingsOnly) {
            // Oracle recursive
            return ' order siblings by ' + orderByColumns;
        }
        else {
            return ' order by ' + orderByColumns;
        }
    }
    _appendCompoundOrderByColumnAlias(entry, columnNames, query, params) {
        const expression = entry.expression;
        const columns = query.__columns;
        if (typeof expression === 'string') {
            const column = (0, SqlBuilder_1.getQueryColumn)(columns, expression);
            if (!column) {
                throw new Error('Column ' + expression + ' included in the order by not found in the select clause');
            }
            return (columnNames.indexOf(expression) + 1) + '';
        }
        else if ((0, values_1.isValueSource)(expression)) {
            const oldSafeTableOrView = this._getSafeTableOrView(params);
            this._setSafeTableOrView(params, undefined);
            const result = this._appendSql(expression, params);
            this._setSafeTableOrView(params, oldSafeTableOrView);
            return result;
        }
        else {
            const oldSafeTableOrView = this._getSafeTableOrView(params);
            this._setSafeTableOrView(params, undefined);
            const result = this._appendRawFragment(expression, params);
            this._setSafeTableOrView(params, oldSafeTableOrView);
            return result;
        }
    }
    _appendCompoundOrderByColumnAliasInsensitive(entry, columnNames, query, params) {
        const collation = this._connectionConfiguration.insesitiveCollation;
        const stringColumn = this._isStringOrderByColumn(entry, query);
        if (!stringColumn) {
            // Ignore the insensitive term, it do nothing
            return this._appendCompoundOrderByColumnAlias(entry, columnNames, query, params);
        }
        else if (collation) {
            return this._appendCompoundOrderByColumnAlias(entry, columnNames, query, params) + ' collate ' + collation;
        }
        else if (collation === '') {
            return this._appendCompoundOrderByColumnAlias(entry, columnNames, query, params);
        }
        else {
            return 'lower(' + this._appendCompoundOrderByColumnAlias(entry, columnNames, query, params) + ')';
        }
    }
    _buildSelectLimitOffset(query, params) {
        let result = super._buildSelectLimitOffset(query, params);
        if (!result && this._isAggregateArrayWrapped(params) && (query.__orderBy || query.__customization?.beforeOrderByItems || query.__customization?.afterOrderByItems)) {
            result += ' offset 0 rows'; // Workaround to force oracle to order the result (if not the order by is ignored)
        }
        return result;
    }
    _buildInsertMultiple(query, params) {
        const multiple = query.__multiple;
        if (!multiple) {
            throw new Error('Exepected a multiple insert');
        }
        if (multiple.length <= 0) {
            return '';
        }
        this._ensureRootQuery(query, params);
        const table = query.__table;
        const oldSafeTableOrView = this._getSafeTableOrView(params);
        const oldFakeNameOf = this._getFakeNamesOf(params);
        this._setFakeNamesOf(params, undefined);
        const customization = query.__customization;
        this._setSafeTableOrView(params, table);
        const returning = !!query.__idColumn || !!query.__columns;
        let insertQuery = '';
        if (this._insertSupportWith) {
            insertQuery += this._buildWith(query, params);
        }
        if (returning) {
            insertQuery += 'begin ';
        }
        else {
            insertQuery = 'insert ';
            if (customization && customization.afterInsertKeyword) {
                insertQuery += this._appendRawFragment(customization.afterInsertKeyword, params) + ' ';
            }
            insertQuery += 'all';
        }
        const shape = query.__shape;
        let columnsInShape;
        if (shape) {
            columnsInShape = {};
            for (let property in shape) {
                const colunName = shape[property];
                if (typeof colunName === 'string') {
                    columnsInShape[colunName] = true;
                }
            }
        }
        for (let i = 0, length = multiple.length; i < length; i++) {
            if (customization && customization.beforeQuery) {
                insertQuery += this._appendRawFragment(customization.beforeQuery, params) + ' ';
            }
            if (returning) {
                insertQuery += 'insert ';
                if (customization && customization.afterInsertKeyword) {
                    insertQuery += this._appendRawFragment(customization.afterInsertKeyword, params) + ' ';
                }
            }
            insertQuery += this._buildInsertOnConflictBeforeInto(query, params);
            insertQuery += 'into ';
            insertQuery += this._appendTableOrViewName(table, params);
            const sets = multiple[i];
            let columns = '';
            const sequences = [];
            for (var columnName in table) {
                if (columnsInShape) {
                    if (columnName in columnsInShape) {
                        continue;
                    }
                }
                else {
                    if (columnName in sets) {
                        continue;
                    }
                }
                const column = (0, Column_1.__getColumnOfObject)(table, columnName);
                if (!column) {
                    continue;
                }
                const columnPrivate = (0, Column_1.__getColumnPrivate)(column);
                if (!columnPrivate.__sequenceName) {
                    continue;
                }
                if (columns) {
                    columns += ', ';
                }
                columns += this._appendSql(column, params);
                sequences.push(columnPrivate.__sequenceName);
            }
            if (shape) { // Follow shape order
                const properties = Object.getOwnPropertyNames(shape);
                for (let i = 0, length = properties.length; i < length; i++) {
                    const property = properties[i];
                    const columnName = shape[property];
                    const column = (0, Column_1.__getColumnOfObject)(table, columnName);
                    if (!column) {
                        // Additional property provided in the value object
                        // Skipped because it is not part of the table
                        // This allows to have more complex objects used in the query
                        continue;
                    }
                    if (!(property in sets)) {
                        // No value set for that property in the shape
                        continue;
                    }
                    if (columns) {
                        columns += ', ';
                    }
                    columns += this._appendRawColumnName(column, params);
                }
            }
            else { // No shape, follow set order
                const properties = Object.getOwnPropertyNames(sets);
                for (let i = 0, length = properties.length; i < length; i++) {
                    const property = properties[i];
                    const column = (0, Column_1.__getColumnOfObject)(table, property);
                    if (!column) {
                        // Additional property provided in the value object
                        // Skipped because it is not part of the table
                        // This allows to have more complex objects used in the query
                        continue;
                    }
                    if (columns) {
                        columns += ', ';
                    }
                    columns += this._appendRawColumnName(column, params);
                }
            }
            insertQuery += ' (' + columns + ')';
            insertQuery += this._buildInsertOutput(query, params);
            let values = '';
            for (let i = 0, length = sequences.length; i < length; i++) {
                const sequenceName = sequences[i];
                if (values) {
                    values += ', ';
                }
                values += this._nextSequenceValue(params, sequenceName);
            }
            if (shape) { // Follow shape order
                const properties = Object.getOwnPropertyNames(shape);
                for (let i = 0, length = properties.length; i < length; i++) {
                    const property = properties[i];
                    const columnName = shape[property];
                    const column = (0, Column_1.__getColumnOfObject)(table, columnName);
                    if (!column) {
                        // Additional property provided in the value object
                        // Skipped because it is not part of the table
                        // This allows to have more complex objects used in the query
                        continue;
                    }
                    if (!(property in sets)) {
                        // No value set for that property in the shape
                        continue;
                    }
                    if (values) {
                        values += ', ';
                    }
                    const value = sets[property];
                    values += this._appendValueForColumn(column, value, params);
                }
            }
            else { // No shape, follow set order
                const properties = Object.getOwnPropertyNames(sets);
                for (let i = 0, length = properties.length; i < length; i++) {
                    const property = properties[i];
                    const column = (0, Column_1.__getColumnOfObject)(table, property);
                    if (!column) {
                        // Additional property provided in the value object
                        // Skipped because it is not part of the table
                        // This allows to have more complex objects used in the query
                        continue;
                    }
                    if (values) {
                        values += ', ';
                    }
                    const value = sets[property];
                    values += this._appendValueForColumn(column, value, params);
                }
            }
            insertQuery += ' values (' + values + ')';
            insertQuery += this._buildInsertOnConflictBeforeReturning(query, params);
            insertQuery += this._buildInsertReturning(query, params);
            if (customization && customization.afterQuery) {
                insertQuery += ' ' + this._appendRawFragment(customization.afterQuery, params);
            }
            if (returning) {
                insertQuery += '; ';
            }
        }
        this._setSafeTableOrView(params, oldSafeTableOrView);
        this._setFakeNamesOf(params, oldFakeNameOf);
        this._resetRootQuery(query, params);
        if (returning) {
            insertQuery += 'end;';
        }
        else {
            insertQuery += ' select ' + multiple.length + ' from dual';
        }
        return insertQuery;
    }
    _buildInsertDefaultValues(query, params) {
        this._ensureRootQuery(query, params);
        const oldSafeTableOrView = this._getSafeTableOrView(params);
        const oldFakeNameOf = this._getFakeNamesOf(params);
        this._setFakeNamesOf(params, undefined);
        const table = query.__table;
        const customization = query.__customization;
        this._setSafeTableOrView(params, table);
        let insertQuery = '';
        if (customization && customization.beforeQuery) {
            insertQuery += this._appendRawFragment(customization.beforeQuery, params) + ' ';
        }
        if (this._insertSupportWith) {
            insertQuery += this._buildWith(query, params);
        }
        insertQuery += 'insert ';
        if (customization && customization.afterInsertKeyword) {
            insertQuery += this._appendRawFragment(customization.afterInsertKeyword, params) + ' ';
        }
        insertQuery += this._buildInsertOnConflictBeforeInto(query, params);
        insertQuery += 'into ';
        insertQuery += this._appendTableOrViewName(table, params);
        let columns = '';
        for (var columnName in table) {
            const column = (0, Column_1.__getColumnOfObject)(table, columnName);
            if (!column) {
                continue;
            }
            if (columns) {
                columns += ', ';
            }
            columns += this._appendRawColumnName(column, params);
        }
        insertQuery += ' (' + columns + ')';
        insertQuery += this._buildInsertOutput(query, params);
        let values = '';
        for (var columnName in table) {
            const column = (0, Column_1.__getColumnOfObject)(table, columnName);
            if (!column) {
                continue;
            }
            if (values) {
                values += ', ';
            }
            const columnPrivate = (0, Column_1.__getColumnPrivate)(column);
            if (columnPrivate.__sequenceName) {
                values += this._nextSequenceValue(params, columnPrivate.__sequenceName);
            }
            else {
                values += 'default';
            }
        }
        insertQuery += ' values (' + values + ')';
        insertQuery += this._buildInsertOnConflictBeforeReturning(query, params);
        insertQuery += this._buildInsertReturning(query, params);
        if (customization && customization.afterQuery) {
            insertQuery += ' ' + this._appendRawFragment(customization.afterQuery, params);
        }
        this._setSafeTableOrView(params, oldSafeTableOrView);
        this._setFakeNamesOf(params, oldFakeNameOf);
        this._resetRootQuery(query, params);
        return insertQuery;
    }
    _buildInsertOutput(_query, _params) {
        return '';
    }
    _buildInsertReturning(query, params) {
        const idColumn = query.__idColumn;
        if (idColumn) {
            this._setContainsInsertReturningClause(params, true);
            return ' returning ' + this._appendSql(idColumn, params) + ' into ' + this._queryRunner.addOutParam(params, ''); // Empty name for the out params, no special name is requiered
        }
        const result = this._buildQueryReturning(query.__columns, params);
        this._setContainsInsertReturningClause(params, !!result);
        return result;
    }
    _buildQueryReturning(queryColumns, params) {
        if (!queryColumns) {
            return '';
        }
        const columns = {};
        (0, SqlBuilder_1.flattenQueryColumns)(queryColumns, columns, '');
        let requireComma = false;
        let result = '';
        for (const property in columns) {
            if (requireComma) {
                result += ', ';
            }
            result += this._appendSql(columns[property], params);
            requireComma = true;
        }
        if (result) {
            result += ' into ';
        }
        requireComma = false;
        for (const property in columns) {
            if (requireComma) {
                result += ', ';
            }
            result += this._queryRunner.addOutParam(params, property);
            requireComma = true;
        }
        if (!result) {
            return '';
        }
        return ' returning ' + result;
    }
    _isNull(params, valueSource) {
        if ((0, Column_1.isColumn)(valueSource)) {
            this._appendRawColumnName(valueSource, params) + ' is null';
        }
        if ((0, values_1.isValueSource)(valueSource)) {
            const valueSourcePrivate = (0, values_2.__getValueSourcePrivate)(valueSource);
            if (valueSourcePrivate.__isBooleanForCondition) {
                if (valueSourcePrivate.__optionalType === 'required') {
                    return this._falseValueForCondition;
                }
                else {
                    // This is a boolean value (0 or 1 from a column) that need to be use in a boolean expression
                    return '(case when ' + this._appendConditionSql(valueSource, params) + ' then 0 when not ' + this._appendConditionSqlParenthesis(valueSource, params) + ' then 0 else 1 end = 1)';
                }
            }
        }
        return this._appendSqlParenthesis(valueSource, params) + ' is null';
    }
    _isNotNull(params, valueSource) {
        if ((0, Column_1.isColumn)(valueSource)) {
            this._appendRawColumnName(valueSource, params) + ' is not null';
        }
        if ((0, values_1.isValueSource)(valueSource)) {
            const valueSourcePrivate = (0, values_2.__getValueSourcePrivate)(valueSource);
            if (valueSourcePrivate.__isBooleanForCondition) {
                if (valueSourcePrivate.__optionalType === 'required') {
                    return this._trueValueForCondition;
                }
                else {
                    // This is a boolean value (0 or 1 from a column) that need to be use in a boolean expression
                    return '(case when ' + this._appendConditionSql(valueSource, params) + ' then 1 when not ' + this._appendConditionSqlParenthesis(valueSource, params) + ' then 1 else 0 end = 1)';
                }
            }
        }
        return this._appendSqlParenthesis(valueSource, params) + ' is not null';
    }
    _is(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        if ((0, Column_1.isColumn)(valueSource) && (0, Column_1.isColumn)(value) && this._hasSameBooleanTypeAdapter(valueSource, value)) {
            return 'decode(' + this._appendRawColumnName(valueSource, params) + ', ' + this._appendRawColumnName(value, params) + ', 1, 0 ) = 1';
        }
        return 'decode(' + this._appendSqlParenthesis(valueSource, params) + ', ' + this._appendValueParenthesis(value, params, columnType, columnTypeName, typeAdapter) + ', 1, 0 ) = 1';
    }
    _isNot(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        if ((0, Column_1.isColumn)(valueSource) && (0, Column_1.isColumn)(value) && this._hasSameBooleanTypeAdapter(valueSource, value)) {
            return 'decode(' + this._appendRawColumnName(valueSource, params) + ', ' + this._appendRawColumnName(value, params) + ', 1, 0 ) = 0';
        }
        return 'decode(' + this._appendSqlParenthesis(valueSource, params) + ', ' + this._appendValueParenthesis(value, params, columnType, columnTypeName, typeAdapter) + ', 1, 0 ) = 0';
    }
    _random(_params) {
        return '(select dbms_random.value from dual)';
    }
    _currentDate(_params) {
        return 'trunc(current_timestamp)';
    }
    _valueWhenNull(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return 'nvl(' + this._appendSql(valueSource, params) + ', ' + this._appendValue(value, params, columnType, columnTypeName, typeAdapter) + ')';
    }
    _divide(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return this._appendSqlParenthesis(valueSource, params) + ' / ' + this._appendValueParenthesis(value, params, this._getMathArgumentType(columnType, columnTypeName, value), this._getMathArgumentTypeName(columnType, columnTypeName, value), typeAdapter);
    }
    _asDouble(params, valueSource) {
        return 'cast(' + this._appendSql(valueSource, params) + ' as float)';
    }
    _log10(params, valueSource) {
        return 'log(' + this._appendSql(valueSource, params) + ', 10)';
    }
    _cbrt(params, valueSource) {
        return 'power(' + this._appendSql(valueSource, params) + ', 3)';
    }
    _getDate(params, valueSource) {
        return 'extract(day from ' + this._appendSql(valueSource, params) + ')';
    }
    _getTime(params, valueSource) {
        return "extract(day from(sys_extract_utc(" + this._appendSql(valueSource, params) + ") - to_timestamp('1970-01-01', 'YYYY-MM-DD'))) * 86400000 + to_number(to_char(sys_extract_utc(" + this._appendSql(valueSource, params) + "), 'SSSSSFF3'))";
    }
    _getFullYear(params, valueSource) {
        return 'extract(year from ' + this._appendSql(valueSource, params) + ')';
    }
    _getMonth(params, valueSource) {
        return 'extract(month from ' + this._appendSql(valueSource, params) + ') - 1';
    }
    _getDay(params, valueSource) {
        // https://riptutorial.com/oracle/example/31649/getting-the-day-of-the-week
        return "mod(trunc(" + this._appendSql(valueSource, params) + ") - trunc(" + this._appendSql(valueSource, params) + ", 'IW') + 1, 7)";
    }
    _getHours(params, valueSource) {
        return 'extract(hour from ' + this._appendSql(valueSource, params) + ')';
    }
    _getMinutes(params, valueSource) {
        return 'extract(minute from ' + this._appendSql(valueSource, params) + ')';
    }
    _getSeconds(params, valueSource) {
        return 'trunc(extract(second from ' + this._appendSql(valueSource, params) + '))';
    }
    _getMilliseconds(params, valueSource) {
        return "to_number(to_char(" + this._appendSql(valueSource, params) + ", 'FF3'))";
    }
    _buildCallProcedure(params, procedureName, procedureParams) {
        let result = 'begin ' + this._escape(procedureName, false) + '(';
        if (procedureParams.length > 0) {
            result += this._appendSql(procedureParams[0], params);
            for (let i = 1, length = procedureParams.length; i < length; i++) {
                result += ', ' + this._appendSql(procedureParams[i], params);
            }
        }
        return result + '); end;';
    }
    _buildCallFunction(params, functionName, functionParams) {
        let result = 'select ' + this._escape(functionName, false) + '(';
        if (functionParams.length > 0) {
            result += this._appendSql(functionParams[0], params);
            for (let i = 1, length = functionParams.length; i < length; i++) {
                result += ', ' + this._appendSql(functionParams[i], params);
            }
        }
        return result + ') from dual';
    }
    _stringConcat(params, separator, value) {
        if (separator === undefined || separator === null) {
            return 'listagg(' + this._appendSql(value, params) + ", ',') within group (order by" + this._appendSql(value, params) + ')';
        }
        else if (separator === '') {
            return 'listagg(' + this._appendSql(value, params) + ') within group (order by' + this._appendSql(value, params) + ')';
        }
        else {
            return 'listagg(' + this._appendSql(value, params) + ', ' + this._appendValue(separator, params, 'string', 'string', undefined) + ') within group (order by' + this._appendSql(value, params) + ')';
        }
    }
    _stringConcatDistinct(params, separator, value) {
        if (separator === undefined || separator === null) {
            return 'listagg(distinct ' + this._appendSql(value, params) + ", ',') within group (order by" + this._appendSql(value, params) + ')';
        }
        else if (separator === '') {
            return 'listagg(distinct ' + this._appendSql(value, params) + ') within group (order by' + this._appendSql(value, params) + ')';
        }
        else {
            return 'listagg(distinct ' + this._appendSql(value, params) + ', ' + this._appendValue(separator, params, 'string', 'string', undefined) + ') within group (order by' + this._appendSql(value, params) + ')';
        }
    }
    _escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) {
        if (typeof value === 'string') {
            value = value.replace(/\[/g, '[[]');
            value = value.replace(/%/g, '[%]');
            value = value.replace(/_/g, '[]');
            return this._appendValue(value, params, columnType, columnTypeName, typeAdapter);
        }
        else {
            return "replace(replace(replace(" + this._appendValue(value, params, columnType, columnTypeName, typeAdapter) + ", '[', '[[]'), '%', '[%]'), '_', '[]')";
        }
    }
    _like(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return this._appendSqlParenthesis(valueSource, params) + ' like ' + this._appendValue(value, params, columnType, columnTypeName, typeAdapter) + " escape '\\'";
    }
    _notLike(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return this._appendSqlParenthesis(valueSource, params) + ' not like ' + this._appendValue(value, params, columnType, columnTypeName, typeAdapter) + " escape '\\'";
    }
    _likeInsensitive(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        const collation = this._connectionConfiguration.insesitiveCollation;
        if (collation) {
            return this._appendSqlParenthesis(valueSource, params) + ' like (' + this._appendValueParenthesis(value, params, columnType, columnTypeName, typeAdapter) + ' collate ' + collation + ") escape '\\'";
        }
        else if (collation === '') {
            return this._appendSqlParenthesis(valueSource, params) + ' like ' + this._appendValueParenthesis(value, params, columnType, columnTypeName, typeAdapter) + " escape '\\'";
        }
        else {
            return 'lower(' + this._appendSql(valueSource, params) + ') like lower(' + this._appendValue(value, params, columnType, columnTypeName, typeAdapter) + ") escape '\\'";
        }
    }
    _notLikeInsensitive(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        const collation = this._connectionConfiguration.insesitiveCollation;
        if (collation) {
            return this._appendSqlParenthesis(valueSource, params) + ' not like (' + this._appendValueParenthesis(value, params, columnType, columnTypeName, typeAdapter) + ' collate ' + collation + ") escape '\\'";
        }
        else if (collation === '') {
            return this._appendSqlParenthesis(valueSource, params) + ' not like ' + this._appendValueParenthesis(value, params, columnType, columnTypeName, typeAdapter) + " escape '\\'";
        }
        else {
            return 'lower(' + this._appendSql(valueSource, params) + ') not like lower(' + this._appendValue(value, params, columnType, columnTypeName, typeAdapter) + ") escape '\\'";
        }
    }
    _startsWith(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return this._appendSqlParenthesis(valueSource, params) + ' like (' + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
    }
    _notStartsWith(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return this._appendSqlParenthesis(valueSource, params) + ' not like (' + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
    }
    _endsWith(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return this._appendSqlParenthesis(valueSource, params) + " like ('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + ") escape '\\'";
    }
    _notEndsWith(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return this._appendSqlParenthesis(valueSource, params) + " not like ('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + ") escape '\\'";
    }
    _startsWithInsensitive(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        const collation = this._connectionConfiguration.insesitiveCollation;
        if (collation) {
            return this._appendSqlParenthesis(valueSource, params) + ' like ((' + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') collate " + collation + ") escape '\\'";
        }
        else if (collation === '') {
            return this._appendSqlParenthesis(valueSource, params) + ' like (' + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
        }
        else {
            return 'lower(' + this._appendSql(valueSource, params) + ') like lower(' + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
        }
    }
    _notStartsWithInsensitive(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        const collation = this._connectionConfiguration.insesitiveCollation;
        if (collation) {
            return this._appendSqlParenthesis(valueSource, params) + ' not like ((' + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') collate " + collation + ") escape '\\'";
        }
        else if (collation === '') {
            return this._appendSqlParenthesis(valueSource, params) + ' not like (' + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
        }
        else {
            return 'lower(' + this._appendSql(valueSource, params) + ') not like lower(' + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
        }
    }
    _endsWithInsensitive(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        const collation = this._connectionConfiguration.insesitiveCollation;
        if (collation) {
            return this._appendSqlParenthesis(valueSource, params) + " like (('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + ") collate " + collation + ") escape '\\'";
        }
        else if (collation === '') {
            return this._appendSqlParenthesis(valueSource, params) + " like ('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + ") escape '\\'";
        }
        else {
            return 'lower(' + this._appendSql(valueSource, params) + ") like lower('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + ") escape '\\'";
        }
    }
    _notEndsWithInsensitive(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        const collation = this._connectionConfiguration.insesitiveCollation;
        if (collation) {
            return this._appendSqlParenthesis(valueSource, params) + " not like (('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + ") collate " + collation + ") escape '\\'";
        }
        else if (collation === '') {
            return this._appendSqlParenthesis(valueSource, params) + " not like ('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + ") escape '\\'";
        }
        else {
            return 'lower(' + this._appendSql(valueSource, params) + ") not like lower('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + ") escape '\\'";
        }
    }
    _contains(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return this._appendSqlParenthesis(valueSource, params) + " like ('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
    }
    _notContains(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        return this._appendSqlParenthesis(valueSource, params) + " not like ('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
    }
    _containsInsensitive(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        const collation = this._connectionConfiguration.insesitiveCollation;
        if (collation) {
            return this._appendSqlParenthesis(valueSource, params) + " like (('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') collate " + collation + ") escape '\\'";
        }
        else if (collation === '') {
            return this._appendSqlParenthesis(valueSource, params) + " like ('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
        }
        else {
            return 'lower(' + this._appendSql(valueSource, params) + ") like lower('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
        }
    }
    _notContainsInsensitive(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        const collation = this._connectionConfiguration.insesitiveCollation;
        if (collation) {
            return this._appendSqlParenthesis(valueSource, params) + " not like (('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') collate " + collation + ") escape '\\'";
        }
        else if (collation === '') {
            return this._appendSqlParenthesis(valueSource, params) + " not like ('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
        }
        else {
            return 'lower(' + this._appendSql(valueSource, params) + ") not like lower('%' || " + this._escapeLikeWildcard(params, value, columnType, columnTypeName, typeAdapter) + " || '%') escape '\\'";
        }
    }
    _in(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        if (Array.isArray(value) && value.length <= 0) {
            return this._falseValueForCondition;
        }
        return super._in(params, valueSource, value, columnType, columnTypeName, typeAdapter);
    }
    _notIn(params, valueSource, value, columnType, columnTypeName, typeAdapter) {
        if (Array.isArray(value) && value.length <= 0) {
            return this._trueValueForCondition;
        }
        return super._notIn(params, valueSource, value, columnType, columnTypeName, typeAdapter);
    }
    _appendAggragateArrayColumns(aggregatedArrayColumns, aggregatedArrayDistinct, params, _query) {
        const distict = aggregatedArrayDistinct ? 'distinct ' : '';
        if ((0, values_1.isValueSource)(aggregatedArrayColumns)) {
            if ((0, values_1.__isUuidValueSource)((0, values_2.__getValueSourcePrivate)(aggregatedArrayColumns)) && this._getUuidStrategy() === 'custom-functions') {
                return 'json_arrayagg(' + distict + 'raw_to_uuid' + this._appendSql(aggregatedArrayColumns, params) + '))';
            }
            return 'json_arrayagg(' + distict + this._appendSql(aggregatedArrayColumns, params) + ')';
        }
        else {
            const columns = {};
            (0, SqlBuilder_1.flattenQueryColumns)(aggregatedArrayColumns, columns, '');
            let result = '';
            for (let prop in columns) {
                if (result) {
                    result += ', ';
                }
                result += "'" + prop + "' value ";
                const column = columns[prop];
                if ((0, values_1.__isUuidValueSource)((0, values_2.__getValueSourcePrivate)(column)) && this._getUuidStrategy() === 'custom-functions') {
                    result += 'raw_to_uuid(' + this._appendSql(column, params) + ')';
                }
                else {
                    result += this._appendSql(column, params);
                }
            }
            return 'json_arrayagg(' + distict + 'json_object(' + result + '))';
        }
    }
    _appendAggragateArrayWrappedColumns(aggregatedArrayColumns, _params, aggregateId) {
        if ((0, values_1.isValueSource)(aggregatedArrayColumns)) {
            if ((0, values_1.__isUuidValueSource)((0, values_2.__getValueSourcePrivate)(aggregatedArrayColumns)) && this._getUuidStrategy() === 'custom-functions') {
                return 'json_arrayagg(raw_to_uuida_' + aggregateId + '_.result))';
            }
            return 'json_arrayagg(a_' + aggregateId + '_.result)';
        }
        else {
            const columns = {};
            (0, SqlBuilder_1.flattenQueryColumns)(aggregatedArrayColumns, columns, '');
            let result = '';
            for (let prop in columns) {
                if (result) {
                    result += ', ';
                }
                result += "'" + prop + "' value ";
                const column = columns[prop];
                if ((0, values_1.__isUuidValueSource)((0, values_2.__getValueSourcePrivate)(column)) && this._getUuidStrategy() === 'custom-functions') {
                    result += 'raw_to_uuid(a_' + aggregateId + '_.' + this._escape(prop, true) + ')';
                }
                else {
                    result += 'a_' + aggregateId + '_.' + this._escape(prop, true);
                }
            }
            return 'json_arrayagg(json_object(' + result + '))';
        }
    }
}
exports.OracleSqlBuilder = OracleSqlBuilder;
/*
 * Source: Combine reserved words from:
 * - Oracle 10g: https://docs.oracle.com/cd/B19306_01/em.102/b40103/app_oracle_reserved_words.htm
 * - Oracle 19: https://docs.oracle.com/en/database/oracle/oracle-database/19/sqlrf/Oracle-SQL-Reserved-Words.html#GUID-55C49D1E-BE08-4C50-A9DD-8593EB925612
 */
const reservedWords = {
    ACCOUNT: true,
    ACTIVATE: true,
    ADMIN: true,
    ADVISE: true,
    AFTER: true,
    ALL_ROWS: true,
    ALLOCATE: true,
    ANALYZE: true,
    ARCHIVE: true,
    ARCHIVELOG: true,
    ARRAY: true,
    AT: true,
    AUTHENTICATED: true,
    AUTHORIZATION: true,
    AUTOEXTEND: true,
    AUTOMATIC: true,
    BACKUP: true,
    BECOME: true,
    BEFORE: true,
    BEGIN: true,
    BFILE: true,
    BITMAP: true,
    BLOB: true,
    BLOCK: true,
    BODY: true,
    CACHE: true,
    CACHE_INSTANCES: true,
    CANCEL: true,
    CASCADE: true,
    CAST: true,
    CFILE: true,
    CHAINED: true,
    CHANGE: true,
    CHAR_CS: true,
    CHARACTER: true,
    CHECKPOINT: true,
    CHOOSE: true,
    CHUNK: true,
    CLEAR: true,
    CLOB: true,
    CLONE: true,
    CLOSE: true,
    CLOSE_CACHED_OPEN_CURSORS: true,
    COALESCE: true,
    COLUMNS: true,
    COMMIT: true,
    COMMITTED: true,
    COMPATIBILITY: true,
    COMPILE: true,
    COMPLETE: true,
    COMPOSITE_LIMIT: true,
    COMPUTE: true,
    CONNECT_TIME: true,
    CONSTRAINT: true,
    CONSTRAINTS: true,
    CONTENTS: true,
    CONTINUE: true,
    CONTROLFILE: true,
    CONVERT: true,
    COST: true,
    CPU_PER_CALL: true,
    CPU_PER_SESSION: true,
    CURRENT_SCHEMA: true,
    CURREN_USER: true,
    CURSOR: true,
    CYCLE: true,
    DANGLING: true,
    DATABASE: true,
    DATAFILE: true,
    DATAFILES: true,
    DATAOBJNO: true,
    DBA: true,
    DBHIGH: true,
    DBLOW: true,
    DBMAC: true,
    DEALLOCATE: true,
    DEBUG: true,
    DEC: true,
    DECLARE: true,
    DEFERRABLE: true,
    DEFERRED: true,
    DEGREE: true,
    DEREF: true,
    DIRECTORY: true,
    DISABLE: true,
    DISCONNECT: true,
    DISMOUNT: true,
    DISTRIBUTED: true,
    DML: true,
    DOUBLE: true,
    DUMP: true,
    EACH: true,
    ENABLE: true,
    END: true,
    ENFORCE: true,
    ENTRY: true,
    ESCAPE: true,
    EXCEPT: true,
    EXCEPTIONS: true,
    EXCHANGE: true,
    EXCLUDING: true,
    EXECUTE: true,
    EXPIRE: true,
    EXPLAIN: true,
    EXTENT: true,
    EXTENTS: true,
    EXTERNALLY: true,
    FAILED_LOGIN_ATTEMPTS: true,
    FALSE: true,
    FAST: true,
    FIRST_ROWS: true,
    FLAGGER: true,
    FLOB: true,
    FLUSH: true,
    FORCE: true,
    FOREIGN: true,
    FREELIST: true,
    FREELISTS: true,
    FULL: true,
    FUNCTION: true,
    GLOBAL: true,
    GLOBALLY: true,
    GLOBAL_NAME: true,
    GROUPS: true,
    HASH: true,
    HASHKEYS: true,
    HEADER: true,
    HEAP: true,
    IDGENERATORS: true,
    IDLE_TIME: true,
    IF: true,
    INCLUDING: true,
    INDEXED: true,
    INDEXES: true,
    INDICATOR: true,
    IND_PARTITION: true,
    INITIALLY: true,
    INITRANS: true,
    INSTANCE: true,
    INSTANCES: true,
    INSTEAD: true,
    INT: true,
    INTERMEDIATE: true,
    ISOLATION: true,
    ISOLATION_LEVEL: true,
    KEEP: true,
    KEY: true,
    KILL: true,
    LABEL: true,
    LAYER: true,
    LESS: true,
    LIBRARY: true,
    LIMIT: true,
    LINK: true,
    LIST: true,
    LOB: true,
    LOCAL: true,
    LOCKED: true,
    LOG: true,
    LOGFILE: true,
    LOGGING: true,
    LOGICAL_READS_PER_CALL: true,
    LOGICAL_READS_PER_SESSION: true,
    MANAGE: true,
    MASTER: true,
    MAX: true,
    MAXARCHLOGS: true,
    MAXDATAFILES: true,
    MAXINSTANCES: true,
    MAXLOGFILES: true,
    MAXLOGHISTORY: true,
    MAXLOGMEMBERS: true,
    MAXSIZE: true,
    MAXTRANS: true,
    MAXVALUE: true,
    MIN: true,
    MEMBER: true,
    MINIMUM: true,
    MINEXTENTS: true,
    MINVALUE: true,
    MLS_LABEL_FORMAT: true,
    MOUNT: true,
    MOVE: true,
    MTS_DISPATCHERS: true,
    MULTISET: true,
    NATIONAL: true,
    NCHAR: true,
    NCHAR_CS: true,
    NCLOB: true,
    NEEDED: true,
    NESTED: true,
    NETWORK: true,
    NEW: true,
    NEXT: true,
    NOARCHIVELOG: true,
    NOCACHE: true,
    NOCYCLE: true,
    NOFORCE: true,
    NOLOGGING: true,
    NOMAXVALUE: true,
    NOMINVALUE: true,
    NONE: true,
    NOORDER: true,
    NOOVERRIDE: true,
    NOPARALLEL: true,
    NOREVERSE: true,
    NORMAL: true,
    NOSORT: true,
    NOTHING: true,
    NUMERIC: true,
    NVARCHAR2: true,
    OBJECT: true,
    OBJNO: true,
    OBJNO_REUSE: true,
    OFF: true,
    OID: true,
    OIDINDEX: true,
    OLD: true,
    ONLY: true,
    OPCODE: true,
    OPEN: true,
    OPTIMAL: true,
    OPTIMIZER_GOAL: true,
    ORGANIZATION: true,
    OSLABEL: true,
    OVERFLOW: true,
    OWN: true,
    PACKAGE: true,
    PARALLEL: true,
    PARTITION: true,
    PASSWORD: true,
    PASSWORD_GRACE_TIME: true,
    PASSWORD_LIFE_TIME: true,
    PASSWORD_LOCK_TIME: true,
    PASSWORD_REUSE_MAX: true,
    PASSWORD_REUSE_TIME: true,
    PASSWORD_VERIFY_FUNCTION: true,
    PCTINCREASE: true,
    PCTTHRESHOLD: true,
    PCTUSED: true,
    PCTVERSION: true,
    PERCENT: true,
    PERMANENT: true,
    PLAN: true,
    PLSQL_DEBUG: true,
    POST_TRANSACTION: true,
    PRECISION: true,
    PRESERVE: true,
    PRIMARY: true,
    PRIVATE: true,
    PRIVATE_SGA: true,
    PRIVILEGE: true,
    PRIVILEGES: true,
    PROCEDURE: true,
    PROFILE: true,
    PURGE: true,
    QUEUE: true,
    QUOTA: true,
    RANGE: true,
    RBA: true,
    READ: true,
    READUP: true,
    REAL: true,
    REBUILD: true,
    RECOVER: true,
    RECOVERABLE: true,
    RECOVERY: true,
    REF: true,
    REFERENCES: true,
    REFERENCING: true,
    REFRESH: true,
    REPLACE: true,
    RESET: true,
    RESETLOGS: true,
    RESIZE: true,
    RESTRICTED: true,
    RETURN: true,
    RETURNING: true,
    REUSE: true,
    REVERSE: true,
    ROLE: true,
    ROLES: true,
    ROLLBACK: true,
    RULE: true,
    SAMPLE: true,
    SAVEPOINT: true,
    SB4: true,
    SCAN_INSTANCES: true,
    SCHEMA: true,
    SCN: true,
    SCOPE: true,
    SD_ALL: true,
    SD_INHIBIT: true,
    SD_SHOW: true,
    SEGMENT: true,
    SEG_BLOCK: true,
    SEG_FILE: true,
    SEQUENCE: true,
    SERIALIZABLE: true,
    SESSION_CACHED_CURSORS: true,
    SESSIONS_PER_USER: true,
    SHARED: true,
    SHARED_POOL: true,
    SHRINK: true,
    SKIP: true,
    SKIP_UNUSABLE_INDEXES: true,
    SNAPSHOT: true,
    SOME: true,
    SORT: true,
    SPECIFICATION: true,
    SPLIT: true,
    SQL_TRACE: true,
    STANDBY: true,
    STATEMENT_ID: true,
    STATISTICS: true,
    STOP: true,
    STORAGE: true,
    STORE: true,
    STRUCTURE: true,
    SWITCH: true,
    SYS_OP_ENFORCE_NOT_NULL$: true,
    SYS_OP_NTCIMG$: true,
    SYSDBA: true,
    SYSOPER: true,
    SYSTEM: true,
    TABLES: true,
    TABLESPACE: true,
    TABLESPACE_NO: true,
    TABNO: true,
    TEMPORARY: true,
    THAN: true,
    THE: true,
    THREAD: true,
    TIMESTAMP: true,
    TIME: true,
    TOPLEVEL: true,
    TRACE: true,
    TRACING: true,
    TRANSACTION: true,
    TRANSITIONAL: true,
    TRIGGERS: true,
    TRUE: true,
    TRUNCATE: true,
    TX: true,
    TYPE: true,
    UB2: true,
    UBA: true,
    UNARCHIVED: true,
    UNDO: true,
    UNLIMITED: true,
    UNLOCK: true,
    UNRECOVERABLE: true,
    UNTIL: true,
    UNUSABLE: true,
    UNUSED: true,
    UPDATABLE: true,
    USAGE: true,
    USE: true,
    USING: true,
    VALIDATION: true,
    VALUE: true,
    VARYING: true,
    WHEN: true,
    WITHOUT: true,
    WORK: true,
    WRITE: true,
    WRITEDOWN: true,
    WRITEUP: true,
    XID: true,
    YEAR: true,
    ZONE: true,
    ACCESS: true,
    ADD: true,
    ALL: true,
    ALTER: true,
    AND: true,
    ANY: true,
    AS: true,
    ASC: true,
    AUDIT: true,
    BETWEEN: true,
    BY: true,
    CHAR: true,
    CHECK: true,
    CLUSTER: true,
    COLUMN: true,
    COLUMN_VALUE: true,
    COMMENT: true,
    COMPRESS: true,
    CONNECT: true,
    CREATE: true,
    CURRENT: true,
    DATE: true,
    DECIMAL: true,
    DEFAULT: true,
    DELETE: true,
    DESC: true,
    DISTINCT: true,
    DROP: true,
    ELSE: true,
    EXCLUSIVE: true,
    EXISTS: true,
    FILE: true,
    FLOAT: true,
    FOR: true,
    FROM: true,
    GRANT: true,
    GROUP: true,
    HAVING: true,
    IDENTIFIED: true,
    IMMEDIATE: true,
    IN: true,
    INCREMENT: true,
    INDEX: true,
    INITIAL: true,
    INSERT: true,
    INTEGER: true,
    INTERSECT: true,
    INTO: true,
    IS: true,
    LEVEL: true,
    LIKE: true,
    LOCK: true,
    LONG: true,
    MAXEXTENTS: true,
    MINUS: true,
    MLSLABEL: true,
    MODE: true,
    MODIFY: true,
    NESTED_TABLE_ID: true,
    NOAUDIT: true,
    NOCOMPRESS: true,
    NOT: true,
    NOWAIT: true,
    NULL: true,
    NUMBER: true,
    OF: true,
    OFFLINE: true,
    ON: true,
    ONLINE: true,
    OPTION: true,
    OR: true,
    ORDER: true,
    PCTFREE: true,
    PRIOR: true,
    PUBLIC: true,
    RAW: true,
    RENAME: true,
    RESOURCE: true,
    REVOKE: true,
    ROW: true,
    ROWID: true,
    ROWNUM: true,
    ROWS: true,
    SELECT: true,
    SESSION: true,
    SET: true,
    SHARE: true,
    SIZE: true,
    SMALLINT: true,
    START: true,
    SUCCESSFUL: true,
    SYNONYM: true,
    SYSDATE: true,
    TABLE: true,
    THEN: true,
    TO: true,
    TRIGGER: true,
    UID: true,
    UNION: true,
    UNIQUE: true,
    UPDATE: true,
    USER: true,
    VALIDATE: true,
    VALUES: true,
    VARCHAR: true,
    VARCHAR2: true,
    VIEW: true,
    WHENEVER: true,
    WHERE: true,
    WITH: true
};
